@@ -4,6 +4,7 @@
 # Description:  Read/write MacOS system tags
 #
 # Revisions:    2017/03/01 - P. Harvey Created
+#               2020/10/13 - PH Added ability to read MacOS "._" files
 #------------------------------------------------------------------------------
 
 package Image::ExifTool::MacOS;
@@ -11,15 +12,38 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.09';
+$VERSION = '1.12';
 
 sub MDItemLocalTime($);
+sub ProcessATTR($$$);
 
 my %mdDateInfo = (
     ValueConv => \&MDItemLocalTime,
     PrintConv => '$self->ConvertDateTime($val)',
 );
 
+# Information decoded from Mac OS sidecar files
+%Image::ExifTool::MacOS::Main = (
+    GROUPS => { 0 => 'File', 1 => 'MacOS' },
+    NOTES => q{
+        Note that on some filesystems, MacOS creates sidecar files with names that
+        begin with "._".  ExifTool will read these files if specified, and extract
+        the information listed in the following table without the need for extra
+        options, but these files are not writable directly.
+    },
+    2 => {
+        Name => 'RSRC',
+        SubDirectory => { TagTable => 'Image::ExifTool::RSRC::Main' },
+    },
+    9 => {
+        Name => 'ATTR',
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::MacOS::XAttr',
+            ProcessProc => \&ProcessATTR,
+        },
+    },
+);
+
 # "mdls" tags (ref PH)
 %Image::ExifTool::MacOS::MDItem = (
     WRITE_PROC => \&Image::ExifTool::DummyWriteProc,
@@ -28,7 +52,7 @@ my %mdDateInfo = (
     NOTES => q{
         MDItem tags are extracted using the "mdls" utility.  They are extracted if
         any "MDItem*" tag or the MacOS group is specifically requested, or by
-        setting the L<MDItemTags|../ExifTool.html#MDItemTags> API option to 1 or the L<RequestAll|../ExifTool.html#RequestAll> API option to 2 or
+        setting the API L<MDItemTags|../ExifTool.html#MDItemTags> option to 1 or the API L<RequestAll|../ExifTool.html#RequestAll> option to 2 or
         higher.  Note that these tags do not necessarily reflect the current
         metadata of a file -- it may take some time for the MacOS mdworker daemon to
         index the file after a metadata change.
@@ -220,7 +244,9 @@ my %mdDateInfo = (
     NOTES => q{
         XAttr tags are extracted using the "xattr" utility.  They are extracted if
         any "XAttr*" tag or the MacOS group is specifically requested, or by setting
-        the L<XAttrTags|../ExifTool.html#XAttrTags> API option to 1 or the L<RequestAll|../ExifTool.html#RequestAll> API option to 2 or higher.
+        the API L<XAttrTags|../ExifTool.html#XAttrTags> option to 1 or the API L<RequestAll|../ExifTool.html#RequestAll> option to 2 or higher.
+        And they are extracted by default from MacOS "._" files when reading
+        these files directly.
     },
     'com.apple.FinderInfo' => {
         Name => 'XAttrFinderInfo',
@@ -325,7 +351,7 @@ sub MDItemLocalTime($)
 
 #------------------------------------------------------------------------------
 # Set MacOS MDItem and XAttr tags from new tag values
-# Inputs: 0) Exiftool ref, 1) file name, 2) list of tags to set
+# Inputs: 0) ExifTool ref, 1) file name, 2) list of tags to set
 # Returns: 1=something was set OK, 0=didn't try, -1=error (and warning set)
 # Notes: There may be errors even if 1 is returned
 sub SetMacOSTags($$$)
@@ -343,7 +369,7 @@ sub SetMacOSTags($$$)
             next unless $overwrite;
             if ($overwrite < 0) {
                 my $operation = $$nvHash{Shift} ? 'Shifting' : 'Conditional replacement';
-                $et->Warn("$operation of $tag not yet supported");
+                $et->Warn("$operation of MacOS $tag not yet supported");
                 next;
             }
         }
@@ -355,7 +381,7 @@ sub SetMacOSTags($$$)
                 $val = Image::ExifTool::ConvertUnixTime($time, 1) if $time;
             }
             $val =~ s{(\d{4}):(\d{2}):(\d{2})}{$2/$3/$1};   # reformat for setfile
-            $cmd = "setfile -d '${val}' '${f}'";
+            $cmd = "/usr/bin/setfile -d '${val}' '${f}'";
         } elsif ($tag eq 'MDItemUserTags') {
             # (tested with "tag" version 0.9.0)
             ($f = $file) =~ s/'/'\\''/g;
@@ -365,7 +391,7 @@ sub SetMacOSTags($$$)
                 my @dels = @{$$nvHash{DelValue}};
                 s/'/'\\''/g foreach @dels;
                 my $del = join ',', @dels;
-                $err = system "tag -r '${del}' '${f}'>/dev/null 2>&1";
+                $err = system "/usr/local/bin/tag -r '${del}' '${f}'>/dev/null 2>&1";
                 unless ($err) {
                     $et->VerboseValue("- $tag", $del);
                     $result = 1;
@@ -377,13 +403,13 @@ sub SetMacOSTags($$$)
                 s/'/'\\''/g foreach @vals;
                 my $opt = $overwrite > 0 ? '-s' : '-a';
                 $val = @vals ? join(',', @vals) : '';
-                $cmd = "tag $opt '${val}' '${f}'";
+                $cmd = "/usr/local/bin/tag $opt '${val}' '${f}'";
                 $et->VPrint(1,"    - $tag = (all)\n") if $overwrite > 0;
                 undef $val if $val eq '';
             }
         } elsif ($tag eq 'XAttrQuarantine') {
             ($f = $file) =~ s/'/'\\''/g;
-            $cmd = "xattr -d com.apple.quarantine '${f}'";
+            $cmd = "/usr/bin/xattr -d com.apple.quarantine '${f}'";
             $silentErr = 256;   # (will get this error if attribute doesn't exist)
         } else {
             ($f = $file) =~ s/(["\\])/\\$1/g;   # escape necessary characters for script
@@ -400,7 +426,7 @@ sub SetMacOSTags($$$)
                 $v = $val ? 8 - $val : 0;       # convert from label to label index (0 for no label)
                 $attr = 'label index';
             }
-            $cmd = qq(osascript -e 'set fp to POSIX file "$f" as alias' -e \\
+            $cmd = qq(/usr/bin/osascript -e 'set fp to POSIX file "$f" as alias' -e \\
                 'tell application "Finder" to set $attr of file fp to "$v"');
         }
         if (defined $cmd) {
@@ -429,7 +455,7 @@ sub ExtractMDItemTags($$)
 
     ($fn = $file) =~ s/([`"\$\\])/\\$1/g;   # escape necessary characters
     $et->VPrint(0, '(running mdls)');
-    my @mdls = `mdls "$fn" 2> /dev/null`;   # get MacOS metadata
+    my @mdls = `/usr/bin/mdls "$fn" 2> /dev/null`;   # get MacOS metadata
     if ($? or not @mdls) {
         $et->Warn('Error running "mdls" to extract MDItem tags');
         return;
@@ -486,8 +512,50 @@ sub ExtractMDItemTags($$)
     $$et{INDENT} =~ s/\| $//;
 }
 
+        
 #------------------------------------------------------------------------------
-# Extract MacOS extended attribute tags
+# Read MacOS XAttr value
+# Inputs: 0) ExifTool object ref, 1) file name
+sub ReadXAttrValue($$$$)
+{
+    my ($et, $tagTablePtr, $tag, $val) = @_;
+    # add to our table if necessary
+    unless ($$tagTablePtr{$tag}) {
+        my $name;
+        # generate tag name from attribute name
+        if ($tag =~ /^com\.apple\.(.*)$/) {
+            ($name = $1) =~ s/^metadata:_?k//;
+            $name =~ s/^metadata:(com_)?//;
+        } else {
+            $name = $tag;
+        }
+        $name =~ s/[.:_]([a-z])/\U$1/g;
+        $name = 'XAttr' . ucfirst $name;
+        my %tagInfo = ( Name => $name );
+        $tagInfo{Groups} = { 2 => 'Time' } if $tag=~/Date$/;
+        $et->VPrint(0, "  [adding $tag]\n");
+        AddTagToTable($tagTablePtr, $tag, \%tagInfo);
+    }
+    if ($val =~ /^bplist0/) {
+        my %dirInfo = ( DataPt => \$val );
+        require Image::ExifTool::PLIST;
+        if (Image::ExifTool::PLIST::ProcessBinaryPLIST($et, \%dirInfo, $tagTablePtr)) {
+            return undef if ref $dirInfo{Value} eq 'HASH';
+            $val = $dirInfo{Value}
+        } else {
+            $et->Warn("Error decoding $$tagTablePtr{$tag}{Name}");
+            return undef;
+        }
+    }
+    if (not ref $val and ($val =~ /\0/ or length($val) > 200) or $tag eq 'XAttrMDLabel') {
+        my $buff = $val;
+        $val = \$buff;
+    }
+    return $val;
+}
+
+#------------------------------------------------------------------------------
+# Read MacOS extended attribute tags using 'xattr' utility
 # Inputs: 0) ExifTool object ref, 1) file name
 sub ExtractXAttrTags($$)
 {
@@ -497,7 +565,7 @@ sub ExtractXAttrTags($$)
 
     ($fn = $file) =~ s/([`"\$\\])/\\$1/g;       # escape necessary characters
     $et->VPrint(0, '(running xattr)');
-    my @xattr = `xattr -lx "$fn" 2> /dev/null`; # get MacOS extended attributes
+    my @xattr = `/usr/bin/xattr -lx "$fn" 2> /dev/null`; # get MacOS extended attributes
     if ($? or not @xattr) {
         $? and $et->Warn('Error running "xattr" to extract XAttr tags');
         return;
@@ -517,39 +585,8 @@ sub ExtractXAttrTags($$)
             $val .= pack('H*', $_);
             next;
         } elsif ($tag and defined $val) {
-            # add to our table if necessary
-            unless ($$tagTablePtr{$tag}) {
-                my $name;
-                # generate tag name from attribute name
-                if ($tag =~ /^com\.apple\.(.*)$/) {
-                    ($name = $1) =~ s/^metadata:_?k//;
-                    $name =~ s/^metadata:(com_)?//;
-                } else {
-                    $name = $tag;
-                }
-                $name =~ s/[.:_]([a-z])/\U$1/g;
-                $name = 'XAttr' . ucfirst $name;
-                my %tagInfo = ( Name => $name );
-                $tagInfo{Groups} = { 2 => 'Time' } if $tag=~/Date$/;
-                $et->VPrint(0, "  [adding $tag]\n");
-                AddTagToTable($tagTablePtr, $tag, \%tagInfo);
-            }
-            if ($val =~ /^bplist0/) {
-                my %dirInfo = ( DataPt => \$val );
-                require Image::ExifTool::PLIST;
-                if (Image::ExifTool::PLIST::ProcessBinaryPLIST($et, \%dirInfo, $tagTablePtr)) {
-                    next if ref $dirInfo{Value} eq 'HASH';
-                    $val = $dirInfo{Value}
-                } else {
-                    $et->Warn("Error decoding $$tagTablePtr{$tag}{Name}");
-                    next;
-                }
-            }
-            if (not ref $val and ($val =~ /\0/ or length($val) > 200) or $tag eq 'XAttrMDLabel') {
-                my $buff = $val;
-                $val = \$buff;
-            }
-            $et->HandleTag($tagTablePtr, $tag, $val);
+            $val = ReadXAttrValue($et, $tagTablePtr, $tag, $val);
+            $et->HandleTag($tagTablePtr, $tag, $val) if defined $val;
             undef $tag;
             undef $val;
         }
@@ -574,7 +611,7 @@ sub GetFileCreateDate($$)
 
     ($fn = $file) =~ s/([`"\$\\])/\\$1/g;   # escape necessary characters
     $et->VPrint(0, '(running stat)');
-    my $time = `stat -f '%SB' -t '%Y:%m:%d %H:%M:%S%z' "$fn" 2> /dev/null`;
+    my $time = `/usr/bin/stat -f '%SB' -t '%Y:%m:%d %H:%M:%S%z' "$fn" 2> /dev/null`;
     if ($? or not $time or $time !~ s/([-+]\d{2})(\d{2})\s*$/$1:$2/) {
         $et->Warn('Error running "stat" to extract FileCreateDate');
         return;
@@ -584,6 +621,84 @@ sub GetFileCreateDate($$)
     delete $$et{SET_GROUP1};
 }
 
+#------------------------------------------------------------------------------
+# Read ATTR metadata from "._" file
+# Inputs: 0) ExifTool ref, 1) dirInfo ref, 2) tag table ref
+# Return: 1 on success
+# (ref https://www.swiftforensics.com/2018/11/the-dot-underscore-file-format.html)
+sub ProcessATTR($$$)
+{
+    my ($et, $dirInfo, $tagTablePtr) = @_;
+    my $dataPt = $$dirInfo{DataPt};
+    my $dataPos = $$dirInfo{DataPos};
+    my $dataLen = length $$dataPt;
+
+    $dataLen >= 58 and $$dataPt =~ /^.{34}ATTR/s or $et->Warn('Invalid ATTR header'), return 0;
+    my $entries = Get32u($dataPt, 66);
+    $et->VerboseDir('ATTR', $entries);
+    # (Note: The RAF is not in $dirInfo because it would break RSRC reading --
+    # the RSCR block uses relative offsets, while the ATTR block uses absolute! grrr!)
+    my $raf = $$et{RAF};
+    my $pos = 70;       # first entry is after ATTR header
+    my $i;
+    for ($i=0; $i<$entries; ++$i) {
+        $pos + 12 > $dataLen and $et->Warn('Truncated ATTR entry'), last;
+        my $off = Get32u($dataPt, $pos);
+        my $len = Get32u($dataPt, $pos + 4);
+        my $n = Get8u($dataPt, $pos + 10);  # number of characters in tag name
+        $pos + 11 + $n > $dataLen and $et->Warn('Truncated ATTR name'), last;
+        $off -= $dataPos;       # convert to relative offset (grrr!)
+        $off < 0 or $off > $dataLen and $et->Warn('Invalid ATTR offset'), last;
+        my $tag = substr($$dataPt, $pos + 11, $n);
+        $tag =~ s/\0+$//;       # remove null terminator
+        # remove random ID after kMDLabel in tag ID
+        $tag =~ s/^com.apple.metadata:kMDLabel_.*/com.apple.metadata:kMDLabel/s;
+        $off + $len > $dataLen and $et->Warn('Truncated ATTR value'), last;
+        my $val = ReadXAttrValue($et, $tagTablePtr, $tag, substr($$dataPt, $off, $len));
+        $et->HandleTag($tagTablePtr, $tag, $val,
+            DataPt  => $dataPt,
+            DataPos => $dataPos,
+            Start   => $off,
+            Size    => $len,
+        ) if defined $val;
+        $pos += (11 + $n + 3) & -4; # step to next entry (on even 4-byte boundary)
+    }
+    return 1;
+}
+
+#------------------------------------------------------------------------------
+# Read information from a MacOS "._" sidecar file
+# Inputs: 0) ExifTool ref, 1) dirInfo ref
+# Returns: 1 on success, 0 if this wasn't a valid "._" file
+# (ref https://www.swiftforensics.com/2018/11/the-dot-underscore-file-format.html)
+sub ProcessMacOS($$)
+{
+    my ($et, $dirInfo) = @_;
+    my $raf = $$dirInfo{RAF};
+    my ($hdr, $buff, $i);
+
+    return 0 unless $raf->Read($hdr, 26) == 26 and $hdr =~ /^\0\x05\x16\x07\0(.)\0\0Mac OS X        /s;
+    my $ver = ord $1;
+    # (extension may be anything, so just echo back the incoming file extension if it exists)
+    $et->SetFileType(undef, undef, $$et{FILE_EXT});
+    $ver == 2 or $et->Warn("Unsupported file version $ver"), return 1;
+    SetByteOrder('MM');
+    my $tagTablePtr = GetTagTable('Image::ExifTool::MacOS::Main');
+    my $entries = Get16u(\$hdr, 0x18);
+    $et->VerboseDir('MacOS', $entries);
+    $raf->Read($hdr, $entries * 12) == $entries * 12 or $et->Warn('Truncated header'), return 1;
+    for ($i=0; $i<$entries; ++$i) {
+        my $pos = $i * 12;
+        my $tag = Get32u(\$hdr, $pos);
+        my $off = Get32u(\$hdr, $pos + 4);
+        my $len = Get32u(\$hdr, $pos + 8);
+        $len > 100000000 and $et->Warn('Record size too large'), last;
+        $raf->Seek($off,0) and $raf->Read($buff,$len) == $len or $et->Warn('Truncated record'), last;
+        $et->HandleTag($tagTablePtr, $tag, undef, DataPt => \$buff, DataPos => $off, Index => $i);
+    }
+    return 1;
+}
+
 1;  # end
 
 __END__
@@ -600,12 +715,13 @@ This module is used by Image::ExifTool
 
 This module contains definitions required by Image::ExifTool to extract
 MDItem* and XAttr* tags on MacOS systems using the "mdls" and "xattr"
-utilities respectively.  Writable tags use "xattr", "setfile" or "osascript"
-for writing.
+utilities respectively.  It also reads metadata directly from the MacOS "_."
+sidecar files that are used on some filesystems to store file attributes. 
+Writable tags use "xattr", "setfile" or "osascript" for writing.
 
 =head1 AUTHOR
 
-Copyright 2003-2020, Phil Harvey (philharvey66 at gmail.com)
+Copyright 2003-2022, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
